/* plucker.c
 *
 * Copyright (C) 2003 by Jason Day
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include "plucker.h"
#include "libplugin.h"
#include "pprefs.h"


static const char RCSID[] = "$Id: plucker.c,v 1.5 2003/06/05 02:13:41 jday Exp $";


/* Local static functions */

/*
 * This is here so we don't leave zombies running around all over the place.
 */
static void plucker_sig_handler (int sig) {
    int status;

    jp_logf (JP_LOG_DEBUG, "Caught SIGCHLD\n");
    wait (&status);
}

/*
 * Unfortunately, the Plucker python distiller prints everything to stderr,
 * so we can't use popen unless we append "2>&1" to the command to redirect
 * stderr to stdin.
 */
int run_distiller_popen (const char *cmd) {
    FILE *fp;
    char buf[1024];

    switch (fork()) {
        case -1:
            jp_logf (JP_LOG_FATAL, "Error forking distiller process: %s\n", strerror (errno));
            return -1;

        case 0:  /* child process */
            dup2 (fileno (stdout), fileno (stderr));
            if ((fp = popen (cmd, "r")) == NULL) {
                jp_logf (JP_LOG_FATAL, "Error popening process: %s\n", strerror (errno));
                _exit (-1);
            }

            while (fgets (buf, 1024, fp)) {
                jp_logf (JP_LOG_GUI, "%s", buf);
            }

            pclose (fp);

            _exit(0);

        default:
            /* don't let zombies wander around */
            signal (SIGCHLD, plucker_sig_handler);
            return 0;
    }
}

/*
 * Runs the distiller by using the given command. We use a customized
 * version of popen to run the distiller, so that we can capture stderr
 * as well as stdout from the process.
 */
int run_distiller (const char *cmd, gboolean block) {
    FILE *fp;
    char buf[1024];
    int status;

    if (!block) {
        switch (fork()) {
            case -1:
                jp_logf (JP_LOG_FATAL, "Error forking distiller process: %s\n", strerror (errno));
                return -1;

            case 0:  /* child process */
                break;

            default:
                /* don't let zombies wander around */
                signal (SIGCHLD, plucker_sig_handler);
                return 0;
        }
    }

    if ((fp = my_popen (cmd)) == NULL) {
        jp_logf (JP_LOG_FATAL, "Error popening process: %s\n", strerror (errno));
        if (!block) {
            _exit (-1);
        }
        return -1;
    }

    while (fgets (buf, 1024, fp)) {
        jp_logf (JP_LOG_GUI, "%s", buf);
    }

    /* my_pclose */
    jp_logf (JP_LOG_DEBUG, "done reading from pipe, waiting\n", cmd);
    wait (&status);

    if (!block) {
        jp_logf (JP_LOG_DEBUG, "child process exiting\n", cmd);
        _exit(0);
    }

    return 0;
}

/*
 * Similar to POSIX popen, but this redirects stderr as well as stdout.
 * It's also much simpler, and only supports reading.
 */
FILE *my_popen (const char *cmd) {
    int fd[2];

    if (pipe (fd) < 0) {
        jp_logf (JP_LOG_FATAL, "Error creating pipe: %s\n", strerror (errno));
        return NULL;
    }

    switch (fork()) {
        case -1:
            jp_logf (JP_LOG_FATAL, "Error forking distiller process: %s\n", strerror (errno));
            close (fd[0]);
            close (fd[1]);
            return NULL;

        case 0:  /* child */
            close (fd[0]);  /* we don't read from pipe */
            dup2 (fd[1], fileno (stdout));
            dup2 (fileno (stdout), fileno (stderr)); /* send stderr to pipe too */

            jp_logf (JP_LOG_DEBUG, "my_popen - execing %s\n", cmd);
            execlp ("/bin/sh", "sh", "-c", cmd, NULL);

            jp_logf (JP_LOG_FATAL, "Error execing %s: %s\n", cmd, strerror (errno));
            close (fd[0]);
            close (fd[1]);
            _exit (127);

        default: /* parent */
            jp_logf (JP_LOG_DEBUG, "my_popen - opening pipe des\n");
            close (fd[1]); /* we don't write to pipe */
            return fdopen (fd[0], "r");
    }
}

/*
 * Runs the distiller in an xterm
 *
 * FIXME -- how do we keep the xterm from closing as soon as the command
 *          is finished?
 */
int run_distiller_in_term (const char *terminal_cmd, const char *cmd, gboolean block) {
    int status;

    switch (fork()) {
        case -1:
            jp_logf (JP_LOG_FATAL, "Error forking distiller process: %s\n", strerror (errno));
            return -1;

        case 0:  /* child process */
            execlp (terminal_cmd, terminal_cmd, "-e", cmd, NULL);
            jp_logf (JP_LOG_FATAL, "Error execing term process: %s\n", strerror (errno));
            _exit (127);

        default:  /* parent */
            if (block) {
                jp_logf (JP_LOG_GUI, "Plucker: waiting for distiller to finish\n");
                wait (&status);
            }
            else {
                /* don't let zombies wander around */
                signal (SIGCHLD, plucker_sig_handler);
            }

            return 0;
    }
}

gboolean should_run_distiller() {
    time_t last_sync;
    time_t ltime;
    struct tm *timep;
    const char *svalue;
    long ivalue;
    int last_year, now_year;
    int last_yday, now_yday;
    int last_hour, now_hour;
    gboolean rval = FALSE;

    /* Get the last sync time from the prefs */
    plucker_get_pref (LAST_SYNC, NULL, &svalue);
    last_sync = (time_t)atol (svalue);
    time (&ltime);
    jp_logf (JP_LOG_DEBUG, "Plucker: last_sync = %ld, now = %ld\n", last_sync, ltime);

    /*
     * Extract the relevant parts of the last sync time and the local time.
     * Note that, despite what the man pages would seem to indicate, the
     * localtime function returns a pointer to a single globally allocated
     * structure which is overwritten with each call.
     */
    timep = localtime (&last_sync);
    last_year = timep->tm_year;
    last_yday = timep->tm_yday;
    last_hour = timep->tm_hour;
    jp_logf (JP_LOG_DEBUG,
             "Plucker: last sync = %d/%d/%d %d:%d:%d\n",
             timep->tm_mon + 1, timep->tm_mday, timep->tm_year + 1900,
             timep->tm_hour, timep->tm_min, timep->tm_sec);

    timep = localtime (&ltime);
    now_year = timep->tm_year;
    now_yday = timep->tm_yday;
    now_hour = timep->tm_hour;
    jp_logf (JP_LOG_DEBUG,
             "Plucker: now = %d/%d/%d %d:%d:%d\n",
             timep->tm_mon + 1, timep->tm_mday, timep->tm_year + 1900,
             timep->tm_hour, timep->tm_min, timep->tm_sec);

    /* Get the sync-when pref, and decide if we need to sync. */
    plucker_get_pref (SYNC_WHEN, &ivalue, NULL);
    switch (ivalue) {
        case EVERY_SYNC:
            rval = TRUE;
            break;

        case HOURLY:
            timep = localtime (&last_sync);
            timep->tm_hour++;
            if (mktime (timep) > ltime) {
                rval = FALSE;
            }
            else {
                rval = TRUE;
            }
            break;

        case MORNING_AND_AFTERNOON:
            /* First, see if the last sync was on the same day */
            if (last_year != now_year) {
                rval = TRUE;
                break;
            }
            if (last_yday != now_yday) {
                rval = TRUE;
                break;
            }

            if (last_hour < 12) {
                /* morning */
                if (now_hour < 12) {
                    rval = FALSE;
                }
                else {
                    rval = TRUE;
                }
            }
            else {
                /* afternoon */
                if (now_hour >= 12) {
                    rval = FALSE;
                }
                else {
                    rval = TRUE;
                }
            }

            break;

        case DAILY:
            timep = localtime (&last_sync);
            timep->tm_mday++;
            if (mktime (timep) > ltime) {
                rval = FALSE;
            }
            else {
                rval = TRUE;
            }
            break;

        case MANUAL:
            rval = FALSE;
            break;

        default:
            jp_logf (JP_LOG_WARN,
                     "Plucker: Unrecognized pref value for sync_when: %d\n",
                     ivalue);
    }

    return rval;
}

