/* plugin.c
 *
 * Copyright (C) 2002 by Jason Day
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <utime.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>

#include <pi-socket.h>
#include <pi-dlp.h>
#include <pi-file.h>

#include "plucker.h"
#include "gui.h"
#include "libplugin.h"
#include "pprefs.h"


static const char RCSID[] = "$Id: plugin.c,v 1.6 2003/06/05 02:13:41 jday Exp $";


/* Local static functions */
static int sync_install(char *filename, int sd);


/* This plugin was designed to work with version 0.99 or greater of jpilot */
void plugin_version (int *major_version, int *minor_version) {
    *major_version = 0;
    *minor_version = 99;
}

int plugin_get_name (char *name, int len) {
    strncpy (name, "jpilot-plucker " VERSION, len);
    return 0;
}

int plugin_get_menu_name (char *name, int len) {
    strncpy (name, "Plucker", len);
    return 0;
}

int plugin_get_help_name (char *name, int len) {
    strncpy(name, "About JPilot-Plucker", len);
    return 0;
}

int plugin_help (char **text, int *width, int *height) {
   *text = strdup(
	   "Plucker plugin for J-Pilot\n"
           "version " VERSION "\n"
	   "by Jason Day (c) 2003.\n"
	   "jasonday@worldnet.att.net\n"
	   "http://jasonday.home.att.net/\n"
	   );

   /* Specifying 0 for width and height lets GTK decide */
   *height = 0;
   *width = 0;

   return 0;
}

int plugin_startup (jp_startup_info *info) {
    jp_init();
    jp_logf (JP_LOG_DEBUG, "Plucker: plugin_startup\n");

    jp_logf (JP_LOG_DEBUG, "Plucker: Loading prefs\n");
    plucker_prefs_init();
    if (plucker_load_prefs() < 0) {
        jp_logf (JP_LOG_WARN, "Plucker: Unable to load preferences file " PREFS_FILE "\n");
    }
    else {
        jp_logf (JP_LOG_DEBUG, "Plucker: loaded preferences from " PREFS_FILE "\n");
    }

    return 0;
}

int plugin_pre_sync() {
    jp_logf (JP_LOG_DEBUG, "Plucker: plugin_pre_sync\n");

    /*
     * Load the prefs file again. Note that we have to do this because the
     * sync process is forked from the main process, so all global data
     * (including the preferences) is just a copy.  Since we save the last
     * sync time in the preferences, we have to reload the prefs file every
     * time we sync.
     */
    if (plucker_load_prefs() < 0) {
        jp_logf (JP_LOG_WARN, "Plucker: Unable to load preferences file " PREFS_FILE "\n");
    }
    else {
        jp_logf (JP_LOG_DEBUG, "Plucker: loaded preferences from " PREFS_FILE "\n");
    }

    return 0;
}

int plugin_sync (int sd) {
    DIR *dir;
    struct dirent *entry;
    long ivalue;
    const char *svalue;
    time_t ltime;
    char path[NAME_MAX];

    /*
     * Running the distiller really belongs in plugin_pre_sync.  Despite the
     * name, however, plugin_pre_sync is called *after* J-Pilot opens the
     * connection to the palm, but the socket descriptor is not passed in.
     * This means we can't call pi_watchdog from plugin_pre_sync, which in
     * turn means that the distiller process will almost certainly cause the
     * palm connection to timeout.  So we do it here.
     */
    if (should_run_distiller()) {
        const gchar *cmd;
        const gchar *terminal_cmd;

        pi_watchdog (sd, 10);   /* prevent the palm from timing out */

        plucker_get_pref (DISTILLER_CMD, 0, &cmd);
        jp_logf (JP_LOG_DEBUG, "distiller command: [%s]\n", cmd);

        plucker_get_pref (USE_TERM, &ivalue, NULL);
        if (ivalue) {
            plucker_get_pref (TERM_CMD, 0, &terminal_cmd);
            jp_logf (JP_LOG_DEBUG, "terminal command: [%s]\n", terminal_cmd);
            run_distiller_in_term (terminal_cmd, cmd, TRUE);
        }
        else {
            run_distiller (cmd, TRUE);
        }

        pi_watchdog (sd, 0);   /* back to normal */
    }

    /* get the "install from" directory */
    plucker_get_pref (INSTALL_DIR, 0, &svalue);
    jp_logf (JP_LOG_GUI, "Plucker: Installing all databases in %s\n", svalue);
    dir = opendir (svalue);
    if (!dir) {
        jp_logf (JP_LOG_WARN, "Plucker: Cannot open directory %s: %s\n", svalue, strerror (errno));
        return -1;
    }

    /* iterate over each file in the directory and look for one with a .pdb
     * extension */
    while ((entry = readdir (dir)) != NULL) {
        char *pc;
        jp_logf (JP_LOG_DEBUG, "Plucker: entry name: %s\n", entry->d_name);

        pc = strstr (entry->d_name, ".pdb");
        if (pc) jp_logf (JP_LOG_DEBUG, "Plucker: pc = [%s], len = [%d]\n", pc, strlen (pc));
        if (pc && (strlen (pc) == 4)) {
            /*
             * FIXME
             * Fetch the DB info and compare the mod times first.
             */
            snprintf (path, NAME_MAX, "%s/%s", svalue, entry->d_name);
            sync_install (path, sd);
        }
    }
    closedir (dir);

    /* update the last sync time */
    time (&ltime);
    svalue = g_strdup_printf ("%ld", ltime);
    jp_logf (JP_LOG_DEBUG, "Plucker: setting last sync time: %s\n", svalue);
    plucker_set_pref (LAST_SYNC, 0, svalue);
    g_free ((char *)svalue);
    plucker_save_prefs();

    jp_logf (JP_LOG_GUI, "Plucker: Sync complete.\n");

    return 0;
}

int plugin_post_sync() {
    jp_logf (JP_LOG_DEBUG, "Plucker: plugin_post_sync\n");

    return 0;
}


/*
 * This function is called by J-Pilot when the user selects this plugin
 * from the plugin menu, or from the search window when a search result
 * record is chosen.  In the latter case, unique ID will be set.  This
 * application should go directly to that record in the case.
 */
int plugin_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
    jp_logf(JP_LOG_DEBUG, "Plucker: plugin gui started, unique_id=%d\n", unique_id);
    return init_gui (vbox, hbox, unique_id);
}

int plugin_gui_cleanup() {
    jp_logf (JP_LOG_DEBUG, "plugin_gui_cleanup()\n");
    return destroy_gui();
}


/*
 * Called when JPilot shuts down.
 */
int plugin_exit_cleanup() {
    jp_logf (JP_LOG_DEBUG, "plugin_exit_cleanup()\n");

    plucker_free_prefs();
    return 0;
}

static int sync_install (char *filename, int sd) {
    struct pi_file *f;
    struct  DBInfo info;
    char *pc;
    char log_entry[256];
    int rval;
    char creator[5];

    jp_logf (JP_LOG_DEBUG, "Plucker: sync_install %s\n", filename);

    pc = strrchr (filename, '/');
    if (!pc) {
        pc = filename;
    }
    else {
        pc++;
    }

    jp_logf(JP_LOG_GUI, "Plucker: Installing %s ", pc);

    f = pi_file_open (filename);
    if (!f) {
        jp_logf (JP_LOG_WARN, "\nUnable to open '%s'!\n", filename);
        return -1;
    }

    bzero (&info, sizeof(info));
    pi_file_get_info (f, &info);
    creator[0] = (info.creator & 0xFF000000) >> 24;
    creator[1] = (info.creator & 0x00FF0000) >> 16;
    creator[2] = (info.creator & 0x0000FF00) >> 8;
    creator[3] = (info.creator & 0x000000FF);
    creator[4] = '\0';
    jp_logf(JP_LOG_GUI, "(Creator ID is '%s')...", creator);

    rval = pi_file_install (f, sd, 0);

    if (rval < 0) {
        g_snprintf (log_entry, 255, "Install %s failed", pc);
        log_entry[255] = '\0';
        dlp_AddSyncLogEntry (sd, log_entry);
        dlp_AddSyncLogEntry (sd, "\n");

        jp_logf (JP_LOG_GUI, "Failed.\n");
        jp_logf (JP_LOG_WARN, log_entry);

        pi_file_close (f);
        return -1;
    }
    else {
        /* the space after the %s is a hack, the last char gets cut off */
        g_snprintf (log_entry, 255, "Installed %s ", pc);
        log_entry[255] = '\0';
        dlp_AddSyncLogEntry (sd, log_entry);
        dlp_AddSyncLogEntry (sd, "\n");;

        jp_logf (JP_LOG_GUI, "OK\n");
    }

    pi_file_close(f);

    return 0;
}
